@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2018 Open Source Applications Foundation.
+ * Copyright (c) 2004-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -50,6 +50,10 @@
 #include "char.h"
 #include "shape.h"
 #include "measureunit.h"
+#include "casemap.h"
+#include "tries.h"
+#include "gender.h"
+#include "bidi.h"
 
 
 /* const variable descriptor */
@@ -206,12 +210,23 @@ static PyObject *PyInit_icu(PyObject *m)
     ver = PyString_FromString(PYICU_VER);
     PyObject_SetAttrString(m, "VERSION", ver); Py_DECREF(ver);
 
+    ver = PyString_FromString(PYICU_ICU_MAX_VER);
+    PyObject_SetAttrString(m, "ICU_MAX_MAJOR_VERSION", ver); Py_DECREF(ver);
+
     ver = PyString_FromString(U_ICU_VERSION);
     PyObject_SetAttrString(m, "ICU_VERSION", ver); Py_DECREF(ver);
 
     ver = PyString_FromString(U_UNICODE_VERSION);
     PyObject_SetAttrString(m, "UNICODE_VERSION", ver); Py_DECREF(ver);
 
+    ver = PyString_FromString(PY_VERSION);
+    PyObject_SetAttrString(m, "PY_VERSION", ver); Py_DECREF(ver);
+
+#ifdef PYPY_VERSION
+    ver = PyString_FromString(PYPY_VERSION);
+    PyObject_SetAttrString(m, "PYPY_VERSION", ver); Py_DECREF(ver);
+#endif
+
     PyObject *module = PyImport_ImportModule("icu");
 
     if (!module)
@@ -251,6 +266,10 @@ static PyObject *PyInit_icu(PyObject *m)
     _init_char(m);
     _init_shape(m);
     _init_measureunit(m);
+    _init_casemap(m);
+    _init_tries(m);
+    _init_gender(m);
+    _init_bidi(m);
 
     return m;
 }
