@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2005-2014 Open Source Applications Foundation.
+ * Copyright (c) 2005-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,7 @@
 
 #include "common.h"
 #include <stdarg.h>
+#include <stdlib.h>
 #include <string.h>
 #include <datetime.h>
 
@@ -172,8 +173,7 @@ EXPORT PyObject *PyUnicode_FromUnicodeString(const UChar *utf16, int len16)
         Py_INCREF(Py_None);
         return Py_None;
     }
-
-#if PY_VERSION_HEX < 0x03030000
+#if PY_VERSION_HEX < 0x03030000 || defined(PYPY_VERSION)
     else if (sizeof(Py_UNICODE) == sizeof(UChar))
         return PyUnicode_FromUnicode((const Py_UNICODE *) utf16, len16);
     else
@@ -342,32 +342,42 @@ EXPORT UnicodeString &PyObject_AsUnicodeString(PyObject *object,
         if (sizeof(Py_UNICODE) == sizeof(UChar))
             string.setTo((const UChar *) PyUnicode_AS_UNICODE(object),
                          (int32_t) PyUnicode_GET_SIZE(object));
-        else
+        else if (sizeof(Py_UNICODE) == sizeof(UChar32))
         {
             int32_t len = (int32_t) PyUnicode_GET_SIZE(object);
             Py_UNICODE *pchars = PyUnicode_AS_UNICODE(object);
 
             string = UnicodeString::fromUTF32((const UChar32 *) pchars, len);
         }
+        else
+            abort();  // we should not get here
 #else
         PyUnicode_READY(object);
 
         switch (PyUnicode_KIND(object)) {
           case PyUnicode_WCHAR_KIND: {  // this code path should be deprecated
-              if (sizeof(Py_UNICODE) == sizeof(UChar))
+              if (SIZEOF_WCHAR_T == sizeof(UChar))
               {
                   Py_ssize_t len;
-                  Py_UNICODE *pchars = PyUnicode_AsUnicodeAndSize(object, &len);
+                  wchar_t *wchars = PyUnicode_AsWideCharString(object, &len);
 
-                  string.setTo((const UChar *) pchars, len);
+                  if (wchars != NULL)
+                  {
+                      string.setTo((const UChar *) wchars, len);
+                      PyMem_Free(wchars);
+                  }
               }
-              else
+              else if (SIZEOF_WCHAR_T == sizeof(UChar32))
               {
                   Py_ssize_t len;
-                  Py_UNICODE *pchars = PyUnicode_AsUnicodeAndSize(object, &len);
+                  wchar_t *wchars = PyUnicode_AsWideCharString(object, &len);
 
-                  string = UnicodeString::fromUTF32(
-                      (const UChar32 *) pchars, len);
+                  if (wchars != NULL)
+                  {
+                      string = UnicodeString::fromUTF32(
+                          (const UChar32 *) wchars, len);
+                      PyMem_Free(wchars);
+                  }
               }
               break;
           }
@@ -553,6 +563,17 @@ int abstract_init(PyObject *self, PyObject *args, PyObject *kwds)
     return -1;
 }
 
+PyObject *abstract_method(PyObject *self, PyObject *args)
+{
+    PyObject *err = Py_BuildValue("(sO)", "calling abstract method on",
+                                  self->ob_type);
+
+    PyErr_SetObject(PyExc_NotImplementedError, err);
+    Py_DECREF(err);
+
+    return NULL;
+}
+
 static PyObject *types;
 
 void registerType(PyTypeObject *type, classid id)
@@ -759,6 +780,56 @@ static UnicodeString *toUnicodeStringArray(PyObject *arg, int *len)
     return NULL;
 }
 
+static int *toIntArray(PyObject *arg, int *len)
+{
+    if (PySequence_Check(arg))
+    {
+        *len = (int) PySequence_Size(arg);
+        int *array = new int[*len + 1];
+
+        for (int i = 0; i < *len; i++) {
+            PyObject *obj = PySequence_GetItem(arg, i);
+
+#if PY_MAJOR_VERSION < 3
+            if (PyInt_Check(obj))
+            {
+                array[i] = PyInt_AsLong(obj);
+                Py_DECREF(obj);
+
+                if (!PyErr_Occurred())
+                    continue;
+            }
+            else if (PyLong_Check(obj))
+            {
+                array[i] = PyLong_AsLong(obj);
+                Py_DECREF(obj);
+
+                if (!PyErr_Occurred())
+                    continue;
+            }
+#else
+            if (PyLong_Check(obj))
+            {
+                array[i] = PyLong_AsLong(obj);
+                Py_DECREF(obj);
+
+                if (!PyErr_Occurred())
+                    continue;
+            }
+#endif
+
+            Py_DECREF(obj);
+            delete[] array;
+
+            return NULL;
+        }
+
+        return array;
+    }
+
+    return NULL;
+}
+
 static double *toDoubleArray(PyObject *arg, int *len)
 {
     if (PySequence_Check(arg))
@@ -948,6 +1019,13 @@ int _parseArgs(PyObject **args, int count, const char *types, ...)
               return -1;
           }
 
+          case 'N':           /* None */
+          {
+              if (arg == Py_None)
+                  break;
+              return -1;
+          }
+
           case 'O':           /* python object of given type */
           {
               PyTypeObject *type = va_arg(list, PyTypeObject *);
@@ -1047,6 +1125,23 @@ int _parseArgs(PyObject **args, int count, const char *types, ...)
                 break;
             return -1;
 
+          case 'H':           /* array of int */
+            if (PySequence_Check(arg))
+            {
+                if (PySequence_Length(arg) > 0)
+                {
+                    PyObject *obj = PySequence_GetItem(arg, 0);
+                    int ok = (PyInt_Check(obj) ||
+                              PyLong_Check(obj));
+                    Py_DECREF(obj);
+                    if (ok)
+                        break;
+                }
+                else
+                    break;
+            }
+            return -1;
+
           case 'L':           /* PY_LONG_LONG */
             if (PyLong_Check(arg) || PyInt_Check(arg))
                 break;
@@ -1236,6 +1331,7 @@ int _parseArgs(PyObject **args, int count, const char *types, ...)
 
           case 'K':           /* python object of any type */
           case 'M':           /* python callable */
+          case 'N':           /* None */
           case 'O':           /* python object of given type */
           {
               PyObject **obj = va_arg(list, PyObject **);
@@ -1355,6 +1451,16 @@ int _parseArgs(PyObject **args, int count, const char *types, ...)
               break;
           }
 
+          case 'H':           /* array of int */
+          {
+              int **array = va_arg(list, int **);
+              int *len = va_arg(list, int *);
+              *array = toIntArray(arg, len);
+              if (!*array)
+                  return -1;
+              break;
+          }
+
           case 'L':           /* PY_LONG_LONG */
           {
               PY_LONG_LONG *l = va_arg(list, PY_LONG_LONG *);
@@ -1403,7 +1509,7 @@ int isUnicodeString(PyObject *arg)
             ISINSTANCE(((t_uobject *) arg)->object, UnicodeString));
 }
 
-int32_t toUChar32(UnicodeString& u, UChar32 *c, UErrorCode& status)
+int32_t toUChar32(UnicodeString &u, UChar32 *c, UErrorCode &status)
 {
 #if U_ICU_VERSION_HEX >= 0x04020000
     return u.toUTF32(c, 1, status);
