@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2005-2018 Open Source Applications Foundation.
+ * Copyright (c) 2005-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -160,6 +160,8 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/uversion.h>
 #include <unicode/ushape.h>
 #include <unicode/ucasemap.h>
+#include <unicode/udata.h>
+#include <unicode/ubidi.h>
 
 #define VERSION_HEX(major, minor, patch) \
   (((major) << 24) | ((minor) << 16) | ((patch) << 8))
@@ -197,10 +199,17 @@ typedef intintobjargproc ssizessizeobjargproc;
 
 #if U_ICU_VERSION_HEX >= 0x04080000
 #include <unicode/alphaindex.h>
+#include <unicode/stringtriebuilder.h>
+#include <unicode/bytestriebuilder.h>
+#include <unicode/bytestrie.h>
+#include <unicode/ucharstriebuilder.h>
+#include <unicode/ucharstrie.h>
+#include <unicode/appendable.h>
 #endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
 #include <unicode/listformatter.h>
+#include <unicode/gender.h>
 #endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
@@ -218,6 +227,10 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/simpleformatter.h>
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+#include <unicode/ubiditransform.h>
+#endif
+
 #if U_ICU_VERSION_HEX >= VERSION_HEX(59, 0, 0)
 #include <unicode/casemap.h>
 #include <unicode/edits.h>
@@ -229,6 +242,20 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/stringoptions.h>
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+#include <unicode/numberrangeformatter.h>
+#include <unicode/ulistformatter.h>
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+#include <unicode/formattedvalue.h>
+#include <unicode/localebuilder.h>
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+#include <unicode/localematcher.h>
+#endif
+
 #if U_ICU_VERSION_HEX < 0x04060000
 
 typedef UClassID classid;
@@ -326,6 +353,7 @@ EXPORT UnicodeString *PyObject_AsUnicodeString(PyObject *object);
 EXPORT UDate PyObject_AsUDate(PyObject *object);
 
 int abstract_init(PyObject *self, PyObject *args, PyObject *kwds);
+PyObject *abstract_method(PyObject *self, PyObject *args);
 
 // helper class, to allow argument parsing with proper cleanup
 class charsArg {
@@ -403,6 +431,11 @@ int _parseArgs(PyObject **args, int count, const char *types, va_list list);
 
 #else
 
+#ifdef __clang__
+#pragma clang diagnostic ignored "-Wvariadic-macros"
+#pragma clang diagnostic ignored "-Wgnu-zero-variadic-macro-arguments"
+#endif
+
 #define parseArgs(args, types, rest...) \
     _parseArgs(((PyTupleObject *)(args))->ob_item, \
                (int) PyObject_Size(args), types, ##rest)
