@@ -1,15 +1,15 @@
 /* ====================================================================
- * Copyright (c) 2004-2018 Open Source Applications Foundation.
+ * Copyright (c) 2004-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions: 
+ * Software is furnished to do so, subject to the following conditions:
  *
  * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software. 
+ * in all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -29,10 +29,19 @@
 #include "format.h"
 #include "macros.h"
 #include "dateformat.h"
+#include "measureunit.h"
 #include "numberformat.h"
 
 #if U_ICU_VERSION_HEX >= 0x04080000
-    DECLARE_CONSTANTS_TYPE(UTimeUnitFormatStyle);
+    DECLARE_CONSTANTS_TYPE(UTimeUnitFormatStyle)
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    DECLARE_CONSTANTS_TYPE(UListFormatterField)
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    DECLARE_CONSTANTS_TYPE(UFieldCategory)
 #endif
 
 /* FieldPosition */
@@ -64,7 +73,7 @@ static PyMethodDef t_fieldposition_methods[] = {
 };
 
 DECLARE_TYPE(FieldPosition, t_fieldposition, UObject, FieldPosition,
-             t_fieldposition_init, NULL);
+             t_fieldposition_init, NULL)
 
 /* ParsePosition */
 
@@ -90,7 +99,7 @@ static PyMethodDef t_parseposition_methods[] = {
 };
 
 DECLARE_TYPE(ParsePosition, t_parseposition, UObject, ParsePosition,
-             t_parseposition_init, NULL);
+             t_parseposition_init, NULL)
 
 /* Format */
 
@@ -107,7 +116,7 @@ static PyMethodDef t_format_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(Format, t_format, UObject, Format, abstract_init, NULL);
+DECLARE_TYPE(Format, t_format, UObject, Format, abstract_init, NULL)
 
 /* MeasureFormat */
 
@@ -119,9 +128,26 @@ public:
 
 static PyObject *t_measureformat_createCurrencyFormat(PyTypeObject *type,
                                                       PyObject *args);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(53, 0, 0)
+static PyObject *t_measureformat_formatMeasure(t_measureformat *self,
+                                               PyObject *args);
+static PyObject *t_measureformat_formatMeasures(t_measureformat *self,
+                                                PyObject *args);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(55, 0, 0)
+static PyObject *t_measureformat_formatMeasurePerUnit(t_measureformat *self,
+                                                      PyObject *args);
+#endif
 
 static PyMethodDef t_measureformat_methods[] = {
     DECLARE_METHOD(t_measureformat, createCurrencyFormat, METH_VARARGS | METH_CLASS),
+#if U_ICU_VERSION_HEX >= VERSION_HEX(53, 0, 0)
+    DECLARE_METHOD(t_measureformat, formatMeasure, METH_VARARGS),
+    DECLARE_METHOD(t_measureformat, formatMeasures, METH_VARARGS),
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(55, 0, 0)
+    DECLARE_METHOD(t_measureformat, formatMeasurePerUnit, METH_VARARGS),
+#endif
     { NULL, NULL, 0, NULL }
 };
 
@@ -137,7 +163,7 @@ static void t_measureformat_dealloc(t_measureformat *self)
 }
 
 DECLARE_TYPE(MeasureFormat, t_measureformat, Format, MeasureFormat,
-             abstract_init, t_measureformat_dealloc);
+             abstract_init, t_measureformat_dealloc)
 
 #if U_ICU_VERSION_HEX >= 0x04020000
 
@@ -162,7 +188,7 @@ static PyMethodDef t_timeunitformat_methods[] = {
 };
 
 DECLARE_TYPE(TimeUnitFormat, t_timeunitformat, MeasureFormat, TimeUnitFormat,
-             t_timeunitformat_init, NULL);
+             t_timeunitformat_init, NULL)
 
 #endif
 
@@ -228,7 +254,7 @@ static PyMethodDef t_messageformat_methods[] = {
 };
 
 DECLARE_TYPE(MessageFormat, t_messageformat, Format, MessageFormat,
-             t_messageformat_init, NULL);
+             t_messageformat_init, NULL)
 
 #if U_ICU_VERSION_HEX >= 0x04000000
 
@@ -261,7 +287,7 @@ static PyMethodDef t_pluralrules_methods[] = {
 };
 
 DECLARE_TYPE(PluralRules, t_pluralrules, UObject, PluralRules,
-             t_pluralrules_init, NULL);
+             t_pluralrules_init, NULL)
 
 /* PluralFormat */
 
@@ -302,7 +328,7 @@ static void t_pluralformat_dealloc(t_pluralformat *self)
 }
 
 DECLARE_TYPE(PluralFormat, t_pluralformat, Format, PluralFormat,
-             t_pluralformat_init, t_pluralformat_dealloc);
+             t_pluralformat_init, t_pluralformat_dealloc)
 
 #endif
 
@@ -333,7 +359,7 @@ static PyMethodDef t_selectformat_methods[] = {
 };
 
 DECLARE_TYPE(SelectFormat, t_selectformat, Format, SelectFormat,
-             t_selectformat_init, NULL);
+             t_selectformat_init, NULL)
 
 #endif
 
@@ -345,17 +371,24 @@ public:
 };
 
 static PyObject *t_listformatter_format(t_listformatter *self, PyObject *arg);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+static PyObject *t_listformatter_formatStringsToValue(t_listformatter *self,
+                                                      PyObject *arg);
+#endif
 static PyObject *t_listformatter_createInstance(PyTypeObject *type,
                                                 PyObject *args);
 
 static PyMethodDef t_listformatter_methods[] = {
     DECLARE_METHOD(t_listformatter, format, METH_O),
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    DECLARE_METHOD(t_listformatter, formatStringsToValue, METH_O),
+#endif
     DECLARE_METHOD(t_listformatter, createInstance, METH_VARARGS | METH_CLASS),
     { NULL, NULL, 0, NULL }
 };
 
 DECLARE_TYPE(ListFormatter, t_listformatter, UObject, ListFormatter,
-             abstract_init, NULL);
+             abstract_init, NULL)
 
 #endif
 
@@ -414,10 +447,93 @@ static void t_simpleformatter_dealloc(t_simpleformatter *self)
 }
 
 DECLARE_TYPE(SimpleFormatter, t_simpleformatter, UMemory, SimpleFormatter,
-             t_simpleformatter_init, t_simpleformatter_dealloc);
+             t_simpleformatter_init, t_simpleformatter_dealloc)
 
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+
+/* ConstrainedFieldPosition */
+
+static int t_constrainedfieldposition_init(t_constrainedfieldposition *self,
+                                           PyObject *args, PyObject *kwds);
+
+static PyObject *t_constrainedfieldposition_constrainCategory(
+    t_constrainedfieldposition *self, PyObject *arg);
+static PyObject *t_constrainedfieldposition_constrainField(
+    t_constrainedfieldposition *self, PyObject *args);
+static PyObject *t_constrainedfieldposition_getCategory(
+    t_constrainedfieldposition *self);
+static PyObject *t_constrainedfieldposition_getField(
+    t_constrainedfieldposition *self);
+static PyObject *t_constrainedfieldposition_getStart(
+    t_constrainedfieldposition *self);
+static PyObject *t_constrainedfieldposition_getLimit(
+    t_constrainedfieldposition *self);
+
+static PyMethodDef t_constrainedfieldposition_methods[] = {
+    DECLARE_METHOD(t_constrainedfieldposition, constrainCategory, METH_O),
+    DECLARE_METHOD(t_constrainedfieldposition, constrainField, METH_VARARGS),
+    DECLARE_METHOD(t_constrainedfieldposition, getCategory, METH_NOARGS),
+    DECLARE_METHOD(t_constrainedfieldposition, getField, METH_NOARGS),
+    DECLARE_METHOD(t_constrainedfieldposition, getStart, METH_NOARGS),
+    DECLARE_METHOD(t_constrainedfieldposition, getLimit, METH_NOARGS),
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_TYPE(ConstrainedFieldPosition, t_constrainedfieldposition, UMemory,
+             ConstrainedFieldPosition, t_constrainedfieldposition_init, NULL)
+
+/* FormattedValue */
+
+class t_formattedvalue : public _wrapper {
+public:
+    FormattedValue *object;
+    ConstrainedFieldPosition cfp;
+};
+
+static PyObject *t_formattedvalue_nextPosition(t_formattedvalue *self,
+                                               PyObject *arg);
+
+static PyMethodDef t_formattedvalue_methods[] = {
+    DECLARE_METHOD(t_formattedvalue, nextPosition, METH_O),
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_TYPE(FormattedValue, t_formattedvalue, UMemory,
+             FormattedValue, abstract_init, NULL)
+
+PyObject *wrap_FormattedValue(FormattedValue *value)
+{
+    using icu::number::FormattedNumber;
+    using icu::number::FormattedNumberRange;
+
+    RETURN_WRAPPED_IF_ISINSTANCE(value, FormattedDateInterval);
+    RETURN_WRAPPED_IF_ISINSTANCE(value, FormattedNumber);
+    RETURN_WRAPPED_IF_ISINSTANCE(value, FormattedList);
+    RETURN_WRAPPED_IF_ISINSTANCE(value, FormattedRelativeDateTime);
+    RETURN_WRAPPED_IF_ISINSTANCE(value, FormattedNumberRange);
+    return wrap_FormattedValue(value, T_OWNED);
+}
+
+/* FormattedList */
+
+class t_formattedlist : public _wrapper {
+public:
+    FormattedList *object;
+    ConstrainedFieldPosition cfp;  // for iterator on t_formattedvalue
+};
+
+static PyMethodDef t_formattedlist_methods[] = {
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_BY_VALUE_TYPE(FormattedList, t_formattedlist, FormattedValue,
+                      FormattedList, abstract_init)
+
+#endif
+
+
 /* FieldPosition */
 
 static int t_fieldposition_init(t_fieldposition *self,
@@ -506,31 +622,8 @@ static PyObject *t_fieldposition_setEndIndex(t_fieldposition *self,
     return PyErr_SetArgsError((PyObject *) self, "setEndIndex", arg);
 }
 
-static PyObject *t_fieldposition_richcmp(t_fieldposition *self, PyObject *arg, int op)
-{
-    int b = 0;
-    FieldPosition *i;
+DEFINE_RICHCMP(FieldPosition, t_fieldposition)
 
-    if (!parseArg(arg, "P", TYPE_CLASSID(FieldPosition), &i))
-    {
-        switch (op) {
-          case Py_EQ:
-          case Py_NE:
-            b = *self->object == *i;
-            if (op == Py_EQ)
-                Py_RETURN_BOOL(b);
-            Py_RETURN_BOOL(!b);
-          case Py_LT:
-          case Py_LE:
-          case Py_GT:
-          case Py_GE:
-            PyErr_SetNone(PyExc_NotImplementedError);
-            return NULL;
-        }
-    }
-
-    return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg);
-}
 
 /* ParsePosition */
 
@@ -601,31 +694,7 @@ static PyObject *t_parseposition_setErrorIndex(t_parseposition *self,
     return PyErr_SetArgsError((PyObject *) self, "setErrorIndex", arg);
 }
 
-static PyObject *t_parseposition_richcmp(t_parseposition *self, PyObject *arg, int op)
-{
-    int b = 0;
-    ParsePosition *i;
-
-    if (!parseArg(arg, "P", TYPE_CLASSID(ParsePosition), &i))
-    {
-        switch (op) {
-          case Py_EQ:
-          case Py_NE:
-            b = *self->object == *i;
-            if (op == Py_EQ)
-                Py_RETURN_BOOL(b);
-            Py_RETURN_BOOL(!b);
-          case Py_LT:
-          case Py_LE:
-          case Py_GT:
-          case Py_GE:
-            PyErr_SetNone(PyExc_NotImplementedError);
-            return NULL;
-        }
-    }
-
-    return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg);
-}
+DEFINE_RICHCMP(ParsePosition, t_parseposition)
 
 
 /* Format */
@@ -749,31 +818,7 @@ static PyObject *t_format_getLocaleID(t_format *self, PyObject *args)
     return PyErr_SetArgsError((PyObject *) self, "getLocaleID", args);
 }
 
-static PyObject *t_format_richcmp(t_format *self, PyObject *arg, int op)
-{
-    int b = 0;
-    Format *i;
-
-    if (!parseArg(arg, "P", TYPE_ID(Format), &i))
-    {
-        switch (op) {
-          case Py_EQ:
-          case Py_NE:
-            b = *self->object == *i;
-            if (op == Py_EQ)
-                Py_RETURN_BOOL(b);
-            Py_RETURN_BOOL(!b);
-          case Py_LT:
-          case Py_LE:
-          case Py_GT:
-          case Py_GE:
-            PyErr_SetNone(PyExc_NotImplementedError);
-            return NULL;
-        }
-    }
-
-    return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg);
-}
+DEFINE_RICHCMP(Format, t_format)
 
 
 /* MeasureFormat */
@@ -815,6 +860,136 @@ static PyObject *t_measureformat_createCurrencyFormat(PyTypeObject *type,
     return PyErr_SetArgsError(type, "createCurrencyFormat", args);
 }
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(53, 0, 0)
+static PyObject *t_measureformat_formatMeasure(t_measureformat *self,
+                                               PyObject *args)
+{
+    Measure *measure;
+    FieldPosition dont_care(FieldPosition::DONT_CARE);
+    FieldPosition *fp;
+    UnicodeString u;
+
+    switch (PyTuple_Size(args)) {
+      case 1:
+        if (!parseArgs(args, "P", TYPE_CLASSID(Measure), &measure))
+        {
+            STATUS_CALL(self->object->formatMeasures(
+                measure, 1, u, dont_care, status));
+            return PyUnicode_FromUnicodeString(&u);
+        }
+        break;
+
+      case 2:
+        if (!parseArgs(args, "PP", TYPE_CLASSID(Measure),
+                       TYPE_CLASSID(FieldPosition), &measure, &fp))
+        {
+            STATUS_CALL(self->object->formatMeasures(
+                measure, 1, u, *fp, status));
+            return PyUnicode_FromUnicodeString(&u);
+        }
+        break;
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "formatMeasure", args);
+}
+
+// Limited to one measure since 'new Measure[len]' is forbidden because
+// Measure() is protected.
+static PyObject *t_measureformat_formatMeasures(t_measureformat *self,
+                                                PyObject *args)
+{
+    Measure **measures = NULL;
+    int len;
+    FieldPosition dont_care(FieldPosition::DONT_CARE);
+    FieldPosition *fp;
+    UnicodeString u;
+
+    switch (PyTuple_Size(args)) {
+      case 1:
+        if (!parseArgs(args, "Q", TYPE_CLASSID(Measure), &measures, &len,
+                       TYPE_CLASSID(Measure)))
+        {
+            if (len != 1)
+            {
+                free(measures);
+                break;
+            }
+
+            STATUS_CALL(
+                {
+                    self->object->formatMeasures(
+                        measures[0], 1, u, dont_care, status);
+                    free(measures);
+                });
+
+            return PyUnicode_FromUnicodeString(&u);
+        }
+        break;
+
+      case 2:
+        if (!parseArgs(args, "QP",
+                       TYPE_CLASSID(Measure), TYPE_CLASSID(FieldPosition),
+                       &measures, &len, TYPE_CLASSID(Measure), &fp))
+        {
+            if (len != 1)
+            {
+                free(measures);
+                break;
+            }
+
+            STATUS_CALL(
+                {
+                    self->object->formatMeasures(
+                        measures[0], 1, u, *fp, status);
+                    free(measures);
+                });
+
+            return PyUnicode_FromUnicodeString(&u);
+        }
+        break;
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "formatMeasures", args);
+}
+
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(55, 0, 0)
+static PyObject *t_measureformat_formatMeasurePerUnit(t_measureformat *self,
+                                                      PyObject *args)
+{
+    Measure *measure;
+    MeasureUnit *unit;
+    FieldPosition *fp;
+    UnicodeString u;
+    FieldPosition dont_care(FieldPosition::DONT_CARE);
+
+    switch (PyTuple_Size(args)) {
+      case 2:
+        if (!parseArgs(args, "PP", TYPE_CLASSID(Measure),
+                       TYPE_CLASSID(MeasureUnit), &measure, &unit))
+        {
+            STATUS_CALL(self->object->formatMeasurePerUnit(
+                *measure, *unit, u, dont_care, status));
+            return PyUnicode_FromUnicodeString(&u);
+        }
+        break;
+      case 3:
+        if (!parseArgs(args, "PPP", TYPE_CLASSID(Measure),
+                       TYPE_CLASSID(MeasureUnit), TYPE_CLASSID(FieldPosition),
+                       &measure, &unit, &fp))
+        {
+            STATUS_CALL(self->object->formatMeasurePerUnit(
+                *measure, *unit, u, *fp, status));
+            return PyUnicode_FromUnicodeString(&u);
+        }
+        break;
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "formatMeasurePerUnit", args);
+}
+#endif
+
 
 #if U_ICU_VERSION_HEX >= 0x04020000
 
@@ -1310,11 +1485,26 @@ static PyObject *t_pluralrules_select(t_pluralrules *self, PyObject *arg)
     UnicodeString u;
     int32_t n;
     double d;
+#if U_ICU_VERSION_HEX >= VERSION_HEX(68, 0, 0)
+    PyObject *formatted;
+#endif
 
     if (!parseArg(arg, "i", &n))
         u = self->object->select(n);
     else if (!parseArg(arg, "d", &d))
         u = self->object->select(d);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(68, 0, 0)
+    else if (!parseArg(arg, "O", &FormattedNumberType_, &formatted))
+    {
+        STATUS_CALL(u = self->object->select(
+            *((t_formattednumber *) formatted)->object, status));
+    }
+    else if (!parseArg(arg, "O", &FormattedNumberRangeType_, &formatted))
+    {
+        STATUS_CALL(u = self->object->select(
+            *((t_formattednumberrange *) formatted)->object, status));
+    }
+#endif
     else
         return PyErr_SetArgsError((PyObject *) self, "select", arg);
 
@@ -1385,32 +1575,7 @@ static PyObject *t_pluralrules_forLocale(PyTypeObject *type, PyObject *arg)
     return PyErr_SetArgsError(type, "forLocale", arg);
 }
 
-static PyObject *t_pluralrules_richcmp(t_pluralrules *self,
-                                       PyObject *arg, int op)
-{
-    int b = 0;
-    PluralRules *rules;
-
-    if (!parseArg(arg, "P", TYPE_CLASSID(PluralRules), &rules))
-    {
-        switch (op) {
-          case Py_EQ:
-          case Py_NE:
-            b = *self->object == *rules;
-            if (op == Py_EQ)
-                Py_RETURN_BOOL(b);
-            Py_RETURN_BOOL(!b);
-          case Py_LT:
-          case Py_LE:
-          case Py_GT:
-          case Py_GE:
-            PyErr_SetNone(PyExc_NotImplementedError);
-            return NULL;
-        }
-    }
-
-    return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg);
-}
+DEFINE_RICHCMP(PluralRules, t_pluralrules)
 
 
 /* PluralFormat */
@@ -1782,6 +1947,27 @@ static PyObject *t_listformatter_format(t_listformatter *self, PyObject *arg)
     return PyErr_SetArgsError((PyObject *) self, "format", arg);
 }
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+static PyObject *t_listformatter_formatStringsToValue(t_listformatter *self,
+                                                      PyObject *arg)
+{
+    UnicodeString *array;
+    int count;
+
+    if (!parseArg(arg, "T", &array, &count))
+    {
+        FormattedList value;
+
+        STATUS_CALL(value = self->object->formatStringsToValue(
+            array, (int32_t) count, status));
+
+        return wrap_FormattedList(value);
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "formatStringsToValue", arg);
+}
+#endif
+
 static PyObject *t_listformatter_createInstance(PyTypeObject *type,
                                                 PyObject *args)
 {
@@ -1970,7 +2156,130 @@ static PyObject *t_simpleformatter_formatStrings(
     return PyErr_SetArgsError((PyObject *) self, "formatStrings", arg);
 }
 
-#endif
+#endif  // ICU >= 57
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+
+/* ConstrainedFieldPosition */
+
+static int t_constrainedfieldposition_init(t_constrainedfieldposition *self,
+                                           PyObject *args, PyObject *kwds)
+{
+    switch (PyTuple_Size(args)) {
+      case 0:
+        self->object = new ConstrainedFieldPosition();
+        self->flags = T_OWNED;
+        return 0;
+    }
+
+    PyErr_SetArgsError((PyObject *) self, "__init__", args);
+    return -1;
+}
+
+static PyObject *t_constrainedfieldposition_constrainCategory(
+    t_constrainedfieldposition *self, PyObject *arg)
+{
+    int32_t category;
+
+    if (!parseArg(arg, "i", &category))
+        self->object->constrainCategory(category);
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *t_constrainedfieldposition_constrainField(
+    t_constrainedfieldposition *self, PyObject *args)
+{
+    int32_t category, field;
+
+    switch (PyTuple_Size(args)) {
+      case 2:
+        if (!parseArgs(args, "ii", &category, &field))
+        {
+            self->object->constrainField(category, field);
+            Py_RETURN_NONE;
+        }
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "constrainField", args);
+}
+
+static PyObject *t_constrainedfieldposition_getCategory(
+    t_constrainedfieldposition *self)
+{
+    return PyInt_FromLong(self->object->getCategory());
+}
+
+static PyObject *t_constrainedfieldposition_getField(
+    t_constrainedfieldposition *self)
+{
+    return PyInt_FromLong(self->object->getField());
+}
+
+static PyObject *t_constrainedfieldposition_getStart(
+    t_constrainedfieldposition *self)
+{
+    return PyInt_FromLong(self->object->getStart());
+}
+
+static PyObject *t_constrainedfieldposition_getLimit(
+    t_constrainedfieldposition *self)
+{
+    return PyInt_FromLong(self->object->getLimit());
+}
+
+/* FormattedValue */
+
+static PyObject *t_formattedvalue_nextPosition(t_formattedvalue *self,
+                                               PyObject *arg)
+{
+    PyObject *fp;
+
+    if (!parseArg(arg, "O", &ConstrainedFieldPositionType_, &fp))
+    {
+        bool b;
+
+        STATUS_CALL(b = self->object->nextPosition(
+            *((t_constrainedfieldposition *) fp)->object, status));
+
+        Py_RETURN_BOOL(b);
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "nextPosition", arg);
+}
+
+static PyObject *t_formattedvalue_iter(t_formattedvalue *self)
+{
+    self->cfp.reset();
+    Py_RETURN_SELF();
+}
+
+static PyObject *t_formattedvalue_iter_next(t_formattedvalue *self)
+{
+    bool b;
+
+    STATUS_CALL(b = self->object->nextPosition(self->cfp, status));
+
+    if (b)
+    {
+        return wrap_ConstrainedFieldPosition(
+            new ConstrainedFieldPosition(self->cfp), T_OWNED);
+    }
+
+    PyErr_SetNone(PyExc_StopIteration);
+    return NULL;
+}
+
+static PyObject *t_formattedvalue_str(t_formattedvalue *self)
+{
+    UnicodeString u;
+
+    STATUS_CALL(u = self->object->toString(status));
+    return PyUnicode_FromUnicodeString(&u);
+}
+
+#endif  // ICU >= 64
+
 
 void _init_format(PyObject *m)
 {
@@ -1992,6 +2301,11 @@ void _init_format(PyObject *m)
     SimpleFormatterType_.tp_as_number = &t_simpleformatter_as_number;
     SimpleFormatterType_.tp_flags |= Py_TPFLAGS_CHECKTYPES;
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    FormattedValueType_.tp_iter = (getiterfunc) t_formattedvalue_iter;
+    FormattedValueType_.tp_iternext = (iternextfunc) t_formattedvalue_iter_next;
+    FormattedValueType_.tp_str = (reprfunc) t_formattedvalue_str;
+#endif
 
     REGISTER_TYPE(FieldPosition, m);
     REGISTER_TYPE(ParsePosition, m);
@@ -2014,6 +2328,11 @@ void _init_format(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(57, 0, 0)
     INSTALL_STRUCT(SimpleFormatter, m);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    INSTALL_STRUCT(ConstrainedFieldPosition, m);
+    INSTALL_STRUCT(FormattedValue, m);
+    INSTALL_STRUCT(FormattedList, m);
+#endif
 
     INSTALL_STATIC_INT(FieldPosition, DONT_CARE);
 
@@ -2026,4 +2345,21 @@ void _init_format(PyObject *m)
     INSTALL_ENUM(UTimeUnitFormatStyle, "FULL", UTMUTFMT_FULL_STYLE);
     INSTALL_ENUM(UTimeUnitFormatStyle, "ABBREVIATED", UTMUTFMT_ABBREVIATED_STYLE);
 #endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    INSTALL_CONSTANTS_TYPE(UListFormatterField, m);
+    INSTALL_ENUM(UListFormatterField, "LITERAL_FIELD", ULISTFMT_LITERAL_FIELD);
+    INSTALL_ENUM(UListFormatterField, "ELEMENT_FIELD", ULISTFMT_ELEMENT_FIELD);
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    INSTALL_CONSTANTS_TYPE(UFieldCategory, m);
+    INSTALL_ENUM(UFieldCategory, "UNDEFINED", UFIELD_CATEGORY_UNDEFINED);
+    INSTALL_ENUM(UFieldCategory, "DATE", UFIELD_CATEGORY_DATE);
+    INSTALL_ENUM(UFieldCategory, "NUMBER", UFIELD_CATEGORY_NUMBER);
+    INSTALL_ENUM(UFieldCategory, "LIST", UFIELD_CATEGORY_LIST);
+    INSTALL_ENUM(UFieldCategory, "RELATIVE_DATETIME", UFIELD_CATEGORY_RELATIVE_DATETIME);
+    INSTALL_ENUM(UFieldCategory, "LIST_SPAN", UFIELD_CATEGORY_LIST_SPAN);
+    INSTALL_ENUM(UFieldCategory, "DATE_INTERVAL_SPAN", UFIELD_CATEGORY_DATE_INTERVAL_SPAN);
+#endif
 }
